@@ -1,3 +1,3 @@
-from .equation_graph import *
+from .mappings_graph import *
 from .lower_graph import *
 from .graph import *
\ No newline at end of file
