@@ -1,11 +1,16 @@
 import logging
-from enum import Enum
+import os
+import sys
+import time
+import types
 
 import numpy as np
+
+from numerous.engine.model.ast_parser.equation_form_graph import function_from_graph_generic, \
+    compiled_function_from_graph_generic_llvm
 from numerous.engine.model.lowering.ast_builder import ASTBuilder
 from numerous.engine.model.graph_representation import EdgeType
-from numerous.engine.model.graph_representation.parser_ast import function_from_graph_generic, \
-    compiled_function_from_graph_generic_llvm
+
 from numerous.engine.model.lowering.llvm_builder import LLVMBuilder
 from numerous.engine.model.lowering.utils import Vardef
 from numerous.engine.model.utils import NodeTypes, recurse_Attribute
@@ -13,15 +18,25 @@ from numerous.engine.variables import VariableType
 from numerous.utils.string_utils import d_u
 
 
-
-
-
 class EquationGenerator:
-    def __init__(self, filename, equation_graph, scope_variables, equations, scoped_equations, temporary_variables,system_tag="", use_llvm=True):
+    def __init__(self, filename, equation_graph, scope_variables, equations, scoped_equations,
+                 temporary_variables, system_tag="", use_llvm=True, imports=None, eq_used=[]):
         self.filename = filename
-        self.system_tag=system_tag
+        self.imports = imports
+        self.system_tag = system_tag
         self.scope_variables = scope_variables
         self.set_variables = {}
+        self.states = []
+        self.llvm_names = {}
+        self.deriv = []
+
+        for ix, (sv_id, sv) in enumerate(self.scope_variables.items()):
+            full_tag = d_u(sv.id)
+            if sv.type == VariableType.STATE:
+                self.states.append(full_tag)
+            elif sv.type == VariableType.DERIVATIVE:
+                self.deriv.append(full_tag)
+
         for k, var in temporary_variables.items():
             if var.type == VariableType.TMP_PARAMETER_SET:
                 self.set_variables.update({k: var})
@@ -48,12 +63,8 @@ class EquationGenerator:
         self.scoped_equations = scoped_equations
         self.temporary_variables = temporary_variables
 
-        self.states = []
-
-        self.deriv = []
         self.values_order = {}
 
-        self.vars_node_id = {}
         self.scope_var_node = {}
         self.scalar_variables = {}
 
@@ -71,18 +82,24 @@ class EquationGenerator:
         if self.llvm:
             self.generated_program = LLVMBuilder(
                 np.ascontiguousarray([x.value for x in self.scope_variables.values()], dtype=np.float64),
-                self.values_order, self.states, self.deriv)
+                self.values_order, self.states, self.deriv, system_tag=self.system_tag)
         else:
             self.generated_program = ASTBuilder(
                 np.ascontiguousarray([x.value for x in self.scope_variables.values()], dtype=np.float64),
-                self.values_order, self.states, self.deriv)
+                self.values_order, self.states, self.deriv, system_tag=self.system_tag)
 
         self.mod_body = []
         # Create a kernel of assignments and calls
 
         self.eq_vardefs = {}
         # Loop over equation functions and generate code for each equation.
-        self._parse_equations(equations)
+
+        used_eq = {}
+
+        for eq_key, eq in equations.items():
+            if eq_key in eq_used:
+                used_eq[eq_key]=eq
+        self._parse_equations(used_eq)
 
         self.all_targeted = []
         self.all_read = []
@@ -90,17 +107,10 @@ class EquationGenerator:
         self.all_read_set_vars = []
 
     def _parse_variable(self, full_tag, sv, sv_id):
-        if not sv_id in self.vars_node_id:
-            self.vars_node_id[full_tag] = full_tag
 
         if full_tag not in self.scope_var_node:
             self.scope_var_node[full_tag] = sv
 
-        if sv.type == VariableType.STATE:
-            self.states.append(self.vars_node_id[full_tag])
-        elif sv.type == VariableType.DERIVATIVE:
-            self.deriv.append(self.vars_node_id[full_tag])
-
         # If a scope_variable is part of a set it should be referenced alone
         if sv.set_var:
             if not sv.set_var.id in self.set_variables:
@@ -117,40 +127,60 @@ class EquationGenerator:
             full_tag = d_u(sv.id)
             self._parse_variable(full_tag, sv, sv_id)
 
-    def get_external_function_name(self,ext_func):
+    def get_external_function_name(self, ext_func):
         if self.llvm:
             return self._llvm_func_name(ext_func)
         return ext_func
+
     def _llvm_func_name(self, ext_func):
-        return ext_func + '_llvm1.<locals>.' + ext_func + '_llvm'
+        return self.llvm_names[ext_func + '_llvm1.<locals>.' + ext_func + '_llvm']
 
     def _parse_equations(self, equations):
-        logging.info('make equations for compilation')
+        logging.info('Making equations for compilation')
+
         for eq_key, eq in equations.items():
-            vardef = Vardef(llvm = self.llvm)
+            vardef = Vardef(llvm=self.llvm)
 
-            func, args, target_ids = function_from_graph_generic(eq[2], eq_key, var_def_=vardef)
-
-            eq[2].lower_graph = None
+            eq.graph.lower_graph = None
             if self.llvm:
-                func_llvm, signature, args, target_ids = compiled_function_from_graph_generic_llvm(eq[2],
-                                                                                                   eq_key,
-                                                                                                   var_def_=Vardef(llvm=self.llvm),
-                                                                                                   compiled_function=True)
-                self.generated_program.add_external_function(func_llvm, signature, len(args), target_ids)
+                func_llvm, signature, args, target_ids = compiled_function_from_graph_generic_llvm(
+                    eq.graph,
+                    imports=self.imports,
+                    var_def_=Vardef(llvm=self.llvm),
+                    compiled_function=True,
+                    replacements=eq.replacements,
+                    replace_name=eq_key
+                )
+                self.llvm_names.update(self.generated_program.add_external_function(func_llvm, signature, len(args), target_ids,
+                                                             replacements=eq.replacements,replace_name=eq_key))
+
             else:
-                self.generated_program.add_external_function(func, None, len(args), target_ids)
+                func, args, target_ids = function_from_graph_generic(eq.graph,
+                                                                     var_def_=vardef, arg_metadata=eq.graph.arg_metadata,
+                                                                     replacements=eq.replacements,
+                                                                     replace_name=eq_key
+                                                                     )
+                self.generated_program.add_external_function(func, None, len(args), target_ids,
+                                                             replacements=eq.replacements,replace_name=eq_key)
 
             vardef.llvm_target_ids = target_ids
             vardef.args_order = args
             self.eq_vardefs[eq_key] = vardef
 
+    def search_in_item_scope(self, var_id, item_id):
+        for var in self.scope_variables.values():
+            ##TODO add namespacecheck
+            if var.item.id == item_id and var.tag == self.scope_variables[var_id].tag:
+                return var.id
+        raise ValueError("No variable found for id {}", var_id)
+
     def _process_equation_node(self, n):
 
         eq_key = self.scoped_equations[self.equation_graph.key_map[n]]
 
         # Define the function to call for this eq
         ext_func = recurse_Attribute(self.equation_graph.get(n, 'func'))
+        item_id = self.equation_graph.get(n, 'item_id')
 
         vardef = self.eq_vardefs[eq_key]
 
@@ -159,20 +189,20 @@ class EquationGenerator:
             self.equation_graph.get_edges_for_node_filter(end_node=n, attr='e_type', val=EdgeType.ARGUMENT))
         # Determine the local arguments names
         args_local = [self.equation_graph.key_map[ae[0]] for i, ae in zip(a_indcs, a_edges) if
-                      not self.equation_graph.edges_attr['arg_local'][i] == 'local']
+                      not self.equation_graph.edges_c[i].arg_local == 'local']
 
         # Determine the local arguments names
-        args_scope_var = [self.equation_graph.edges_attr['arg_local'][i] for i, ae in zip(a_indcs, a_edges) if
-                          not self.equation_graph.edges_attr['arg_local'][i] == 'local']
+        args_scope_var = [self.equation_graph.edges_c[i].arg_local for i, ae in zip(a_indcs, a_edges) if
+                          not self.equation_graph.edges_c[i].arg_local == 'local']
 
         # Find the targets by looking for target edges
         t_indcs, t_edges = list(
             self.equation_graph.get_edges_for_node_filter(start_node=n, attr='e_type', val=EdgeType.TARGET))
         targets_local = [self.equation_graph.key_map[te[1]] for i, te in zip(t_indcs, t_edges) if
-                         not self.equation_graph.edges_attr['arg_local'][i] == 'local']
-        targets_scope_var = [self.equation_graph.edges_attr['arg_local'][i] for i, ae in zip(t_indcs, t_edges)
+                         not self.equation_graph.edges_c[i].arg_local == 'local']
+        targets_scope_var = [self.equation_graph.edges_c[i].arg_local for i, ae in zip(t_indcs, t_edges)
                              if
-                             not self.equation_graph.edges_attr['arg_local'][i] == 'local']
+                             not self.equation_graph.edges_c[i].arg_local == 'local']
         set_size = 0
         # Record targeted and read variables
         if self.equation_graph.get(n, 'vectorized'):
@@ -182,8 +212,8 @@ class EquationGenerator:
                           zip(args_scope_var + targets_scope_var, args_local + targets_local)}
 
             # Put the information of args and targets in the scope_var attr of the graph node for those equation
-            self.equation_graph.nodes_attr['scope_var'][n] = {'args': [scope_vars[a] for a in vardef.args],
-                                                              'targets': [scope_vars[a] for a in vardef.targets]}
+            self.equation_graph.nodes[n].scope_var = {'args': [scope_vars[a] for a in vardef.args],
+                                                      'targets': [scope_vars[a] for a in vardef.targets]}
             # Record all targeted variables
             for t in vardef.targets:
                 self.all_targeted_set_vars.append(scope_vars[t])
@@ -198,8 +228,8 @@ class EquationGenerator:
                           zip(args_scope_var + targets_scope_var, args_local + targets_local)}
 
             # Put the information of args and targets in the scope_var attr of the graph node for those equation
-            self.equation_graph.nodes_attr['scope_var'][n] = {'args': [scope_vars[a] for a in vardef.args],
-                                                              'targets': [scope_vars[a] for a in vardef.targets]}
+            self.equation_graph.nodes[n].scope_var = {'args': [scope_vars[a] for a in vardef.args],
+                                                      'targets': [scope_vars[a] for a in vardef.targets]}
             for a in vardef.args:
                 if (sva := scope_vars[a]) in self.set_variables:
                     self.all_read_set_vars.append(sva)
@@ -209,7 +239,7 @@ class EquationGenerator:
             self.all_targeted += [scope_vars[t] for t in
                                   vardef.targets]
 
-        # Generate ast for this equation call
+        # Generate ast for this equation callcompiled_function_from_graph_generic_llvm
         if self.equation_graph.get(n, 'vectorized'):
 
             llvm_args = []
@@ -221,16 +251,17 @@ class EquationGenerator:
                 llvm_args.append(llvm_args_)
             ##reshape to correct format
             llvm_args = [list(x) for x in zip(*llvm_args)]
-            self.generated_program.add_set_call(self.get_external_function_name(ext_func), llvm_args, vardef.llvm_target_ids)
+            self.generated_program.add_set_call(self.get_external_function_name(ext_func), llvm_args,
+                                                vardef.llvm_target_ids)
         else:
             # Generate llvm arguments
             args = []
+
             for a in vardef.args_order:
                 if a in scope_vars:
                     args.append(d_u(scope_vars[a]))
                 else:
-                    args.append(a)
-
+                    args.append(self.search_in_item_scope(a, item_id))
 
             # Add this eq to the llvm_program
             self.generated_program.add_call(self.get_external_function_name(ext_func), args, vardef.llvm_target_ids)
@@ -258,7 +289,7 @@ class EquationGenerator:
             for v_ix, v in zip(v_indcs, value_edges):
                 if (nt := self.equation_graph.get(v[0], 'node_type')) == NodeTypes.VAR or nt == NodeTypes.TMP:
 
-                    if (mix := self.equation_graph.edges_attr['mappings'][v_ix]) == ':':
+                    if (mix := self.equation_graph.edges_c[v_ix].mappings) == ':':
                         mappings[':'].append(self.equation_graph.key_map[v[0]])
 
                     elif isinstance(mix, list):
@@ -271,7 +302,7 @@ class EquationGenerator:
 
                     else:
                         raise ValueError(
-                            f'mapping indices not specified!{self.equation_graph.edges_attr["mappings"][v_ix]}, {self.equation_graph.key_map[t]} <- {self.equation_graph.key_map[v[0]]}')
+                            f'mapping indices not specified!{self.equation_graph.edges_c[v_ix].mappings}, {self.equation_graph.key_map[t]} <- {self.equation_graph.key_map[v[0]]}')
 
                 else:
                     raise ValueError(f'this must be a mistake {self.equation_graph.key_map[v[0]]}')
@@ -335,7 +366,7 @@ class EquationGenerator:
                 else:
                     self.all_read.append(self.equation_graph.key_map[v[0]])
 
-                maps = self.equation_graph.edges_attr['mappings'][vi]
+                maps = self.equation_graph.edges_c[vi].mappings
 
                 if maps == ':':
                     if self.equation_graph.key_map[t] in self.set_variables:
@@ -367,10 +398,10 @@ class EquationGenerator:
                                 mapping_dict[target_var] = [self.set_variables[var_name].get_var_by_idx(v[1]).id]
                         else:
                             raise ValueError(f'Variable  {var_name} mapping not found')
-            for k,v in mapping_dict.items():
+            for k, v in mapping_dict.items():
                 self.generated_program.add_mapping(v, [k])
 
-    def generate_equations(self):
+    def generate_equations(self, export_model=False, clonable=False):
         logging.info('Generate kernel')
         # Generate the ast for the python kernel
         for n in self.topo_sorted_nodes:
@@ -388,25 +419,26 @@ class EquationGenerator:
 
         deriv_idx = []
         state_idx = []
+        save_to_file = export_model or clonable
         for k, v in self.values_order.items():
             if k in self.deriv:
                 deriv_idx.append(v)
             if k in self.states:
                 state_idx.append(v)
         if self.llvm:
-            logging.info('generating llvm')
-            diff, var_func, var_write = self.generated_program.generate(system_tag=self.system_tag,save_opt=True)
+            logging.info('Generating llvm')
+            diff, var_func, var_write = self.generated_program.generate(imports=self.imports,
+                                                                        system_tag=self.system_tag,
+                                                                        save_to_file=save_to_file)
 
             return diff, var_func, var_write, self.values_order, self.scope_variables, np.array(state_idx,
-                                                                                                dtype=np.int64), np.array(
-                deriv_idx, dtype=np.int64)
+                                                                                                dtype=np.int64), \
+                   np.array(
+                       deriv_idx, dtype=np.int64)
         else:
-            self.generated_program.generate(system_tag=self.system_tag)
-            exec('from tmp.listings.'+self.system_tag+'_kernel import *', globals())
-            def var_func():
-                return kernel_variables
-            def var_write(value,idx):
-                np.put(kernel_variables, [idx], value)
-            return global_kernel, var_func, var_write, self.values_order, self.scope_variables, np.array(state_idx,
-                                                                                                dtype=np.int64), np.array(
-                deriv_idx, dtype=np.int64)
\ No newline at end of file
+
+            global_kernel, var_func, var_write = self.generated_program.generate(self.imports,
+                                                                                 system_tag=self.system_tag,
+                                                                                 save_to_file=save_to_file)
+            return global_kernel, var_func, var_write, self.values_order, self.scope_variables, \
+                   np.array(state_idx, dtype=np.int64), np.array(deriv_idx, dtype=np.int64)
\ No newline at end of file
