@@ -1,21 +1,20 @@
 import os
 
 class Historian:
-    def __init__(self, max_size):
+    def __init__(self, max_size, timesteps_reserved_per_event=100):
         self.max_size = max_size
+        self.timesteps_reserved_per_event = timesteps_reserved_per_event
 
-    def get_historian_max_size(self, number_of_timesteps):
+    def get_historian_max_size(self, number_of_timesteps, events_count):
         if not self.max_size:
             ## +1 here since we are using <= to compare inside compiled model
-            return number_of_timesteps+1
+            return number_of_timesteps + events_count * self.timesteps_reserved_per_event + 1
         else:
             return self.max_size
 
     def store(self, df):
         pass
 
-    def need_to_correct(self):
-        return False
 
 
 class InMemoryHistorian(Historian):
@@ -23,10 +22,6 @@ class InMemoryHistorian(Historian):
     def __init__(self):
         super().__init__(None)
 
-    ## REQUIRED WHEN max_size ARE None
-    def need_to_correct(self):
-        return True
-
 
 class LocalHistorian(Historian):
 
@@ -38,4 +33,4 @@ class LocalHistorian(Historian):
         if os.path.isfile(self.filename):
             df.dropna().to_csv(self.filename, mode='a', header=False)
         else:
-            df.dropna().to_csv(self.filename)
+            df.dropna().to_csv(self.filename)
\ No newline at end of file
